package paneles;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.time.LocalDate;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

import tallerVehiculos.Diesel;
import tallerVehiculos.Gasolina;
import tallerVehiculos.Vehiculo;

@SuppressWarnings("serial")
public class PanelDatosVehiculo extends JPanel{
	
	public boolean editando;
	private Vehiculo vehiculoEditable;
	//Atributos comunes de Vehiculos
	private JTextField campoPlaca;
	private JTextField campoVIN;
	private JTextField campoMarca;
	private JTextField campoModelo;
	private JComboBox<String> comboAnio;
	private JTextField campoTamanioMotor;
	private JComboBox<String> comboTraccion;
	//Atributos para Gasolina
	private JComboBox<String> comboTipoFlujo;
	private JTextField campoBobina;
	//Atributos para Diesel
	private JTextField campoTipoBomba;
	private JTextField campoTipoFiltro;
	//botones
	public JButton botonAceptar;
	public JButton botonCancelar;
	//RadioButton para escoger tipo de Vehiculo
	private JRadioButton radioGasolina;
	private JRadioButton radioDiesel;
	
	public PanelDatosVehiculo() {
		iniciarComponentes();
		setLayout(new BorderLayout());
		
		JPanel centro = new JPanel();
		centro.setLayout(new BoxLayout(centro, BoxLayout.Y_AXIS));
		centro.add(new PanelDatosComunes());
		centro.add(new PanelTipoVehiculo());
		centro.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		
		JPanel sur = new JPanel();
		sur.setLayout(new BoxLayout(sur, BoxLayout.X_AXIS));
		JPanel surIzq = new JPanel();
		surIzq.add(botonAceptar);
		JPanel surDer = new JPanel();
		surDer.add(botonCancelar);
		sur.add(surIzq);
		sur.add(surDer);
		
		add(centro, BorderLayout.CENTER);
		add(sur, BorderLayout.SOUTH);
		sur.setBorder(BorderFactory.createEmptyBorder(0, 10, 10, 10));
	}
	
	private void iniciarComponentes() {
		campoPlaca = new JTextField(10);
		campoVIN = new JTextField(10);
		campoMarca = new JTextField(6);
		campoModelo = new JTextField(6);
		//Aceptamos coches desde el año 1920 hasta año actual
		comboAnio = new JComboBox<String>();
		for (int i = LocalDate.now().getYear(); i >= 1920; i--)
			comboAnio.addItem(Integer.toString(i));
		campoTamanioMotor = new JTextField(6);
		//Solo hay dos tipos de traccion para escoger
		comboTraccion = new JComboBox<String>(new String[] {"4x2", "4x4"});
		//Solo dos tipos de flujo de combustible para vehiculos Gasolina
		comboTipoFlujo = new JComboBox<String>(new String[] {"Carburador", "Inyección"});
		campoBobina = new JTextField(8);
		campoTipoBomba = new JTextField(8);
		campoTipoFiltro = new JTextField(8);
		
		botonAceptar = new JButton("Aceptar");
		botonCancelar = new JButton("Cancelar");
		
		radioGasolina = new JRadioButton("Gasolina", true); //Seleccionado por defecto
		seleccionarGasolina();
		radioGasolina.addActionListener(new AccionElegirGasolina());
		radioDiesel = new JRadioButton("Diesel");
		radioDiesel.addActionListener(new AccionElegirDiesel());
		ButtonGroup grupo = new ButtonGroup();
		grupo.add(radioGasolina);
		grupo.add(radioDiesel);
	}
	
	private void seleccionarGasolina() {
		//Activamos campos de Gasolina
		comboTipoFlujo.setEnabled(true);
		campoBobina.setEditable(true);
		//Desactivamos campos Diesel
		campoTipoBomba.setEditable(false);
		campoTipoFiltro.setEditable(false);
	}
	
	private void seleccionarDiesel() {
		//Desactivamos campos de Gasolina
		comboTipoFlujo.setEnabled(false);
		campoBobina.setEditable(false);
		//Activamos campos Diesel
		campoTipoBomba.setEditable(true);
		campoTipoFiltro.setEditable(true);
	}
	
	public void resetCampos() {
		editando = false;
		campoPlaca.setText(null);
		campoVIN.setText(null);
		campoMarca.setText(null);
		campoModelo.setText(null);
		comboAnio.setSelectedIndex(0);
		campoTamanioMotor.setText(null);
		comboTraccion.setSelectedIndex(0);
		comboTipoFlujo.setSelectedIndex(0);
		campoBobina.setText(null);
		campoTipoBomba.setText(null);
		campoTipoFiltro.setText(null);
		modoEdicion(false);
	}
	
	public boolean formularioEsValido() {
		if (!campoPlaca.getText().isEmpty() &&
				!campoVIN.getText().isEmpty() &&
				!campoMarca.getText().isEmpty() &&
				!campoModelo.getText().isEmpty() &&
				!campoTamanioMotor.getText().isEmpty()) {
			//Comprobamos campos específicos
			if (radioGasolina.isSelected())
				return !campoBobina.getText().isEmpty();
			else //Es diesel
				return (!campoTipoBomba.getText().isEmpty() &&
						!campoTipoFiltro.getText().isEmpty());
		}
		else
			return false;
	}
	
	public Vehiculo getNuevoVehiculo() {
		String placa = campoPlaca.getText();
		String VIN = campoVIN.getText();
		String marca = campoMarca.getText();
		String modelo = campoModelo.getText();
		String anio = (String)comboAnio.getSelectedItem();
		String tamanioMotor = campoTamanioMotor.getText();
		String traccion = (String)comboTraccion.getSelectedItem();
		//Tenemos datos comunes. Ahora vemos si es Gasolina o Diesel
		if (radioGasolina.isSelected()) {
			String flujo = (String)comboTipoFlujo.getSelectedItem();
			String bobina = campoBobina.getText();
			return new Gasolina(placa, VIN, marca, modelo, anio, tamanioMotor,
					traccion, flujo, bobina);
		}
		else {//Diesel
			String bomba = campoTipoBomba.getText();
			String filtro = campoTipoFiltro.getText();
			return new Diesel(placa, VIN, marca, modelo, anio, tamanioMotor,
					traccion, bomba, filtro);
		}
	}
	
	public void modoEdicion(boolean modo) {
		if (modo) {
			editando = true;
			campoVIN.setEditable(false);//Dato no modificable
			//RadioButton tampoco son editables
			radioGasolina.setEnabled(false);
			radioDiesel.setEnabled(false);
		}
		else {
			editando = false;
			campoVIN.setEditable(true);
			radioGasolina.setEnabled(true);
			radioDiesel.setEnabled(true);
		}
	}
	
	public void editarVehiculo(Vehiculo v) {
		modoEdicion(true);
		vehiculoEditable = v; //Guardamos referencia
		//Mostramos datos comunes
		campoPlaca.setText(v.getPlaca());
		campoVIN.setText(v.getVIN());
		campoMarca.setText(v.getMarca());
		campoModelo.setText(v.getModelo());
		comboAnio.setSelectedItem(v.getAnio());
		campoTamanioMotor.setText(v.getTamanioMotor());
		comboTraccion.setSelectedItem(v.getTraccion());
		//Ahora depende de si es Diesel o Gasolina
		if (v instanceof Gasolina) {
			radioGasolina.setSelected(true);
			seleccionarGasolina();
			comboTipoFlujo.setSelectedItem((((Gasolina) v).getFlujoGasolina()));
			campoBobina.setText(((Gasolina)v).getTipoBobina());
		}
		else {
			radioDiesel.setSelected(true);
			seleccionarDiesel();
			campoTipoBomba.setText(((Diesel)v).getTipoBomba());
			campoTipoFiltro.setText(((Diesel)v).getTipoFiltro());
		}
	}
	
	public void aceptarEdicion() {
		//Datos comunes
		vehiculoEditable.setPlaca(campoPlaca.getText());
		vehiculoEditable.setMarca(campoMarca.getText());
		vehiculoEditable.setModelo(campoModelo.getText());
		vehiculoEditable.setAnio((String)comboAnio.getSelectedItem());
		vehiculoEditable.setTamanioMotor(campoTamanioMotor.getText());
		vehiculoEditable.setTraccion((String)comboTraccion.getSelectedItem());
		//Ahora segun si Diesel o Gasolina
		if (vehiculoEditable instanceof Gasolina) {
			((Gasolina)vehiculoEditable).setFlujoGasolina((String)comboTipoFlujo.getSelectedItem());
			((Gasolina)vehiculoEditable).setTipoBobina(campoBobina.getText());
		}
		else {
			((Diesel)vehiculoEditable).setTipoBomba(campoTipoBomba.getText());
			((Diesel)vehiculoEditable).setTipoFiltro(campoTipoFiltro.getText());
		}
	}
	
	class PanelDatosComunes extends JPanel {
		public PanelDatosComunes() {
			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			
			JPanel panel1 = new JPanel();
			panel1.add(new PanelEtiqYCompon("Nº Placa: ", campoPlaca));
			panel1.add(new PanelEtiqYCompon("Nº VIN: ", campoVIN));
			JPanel panel2 = new JPanel();
			panel2.add(new PanelEtiqYCompon("Marca: ", campoMarca));
			panel2.add(new PanelEtiqYCompon("Modelo: ", campoModelo));
			panel2.add(new PanelEtiqYCompon("Año: ", comboAnio));
			JPanel panel3 = new JPanel();
			panel3.add(new PanelEtiqYCompon("Tamaño Motor: ", campoTamanioMotor));
			panel3.add(new PanelEtiqYCompon("Tipo de Tracción: ", comboTraccion));
			
			add(panel1); add(panel2); add(panel3);
			
			setBorder(BorderFactory.createCompoundBorder(
					BorderFactory.createTitledBorder("Datos Vehículo"),
					BorderFactory.createEmptyBorder(5, 5, 5, 5)));
		}
	}
	
	class PanelTipoVehiculo extends JPanel {
		public PanelTipoVehiculo() {
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			
			JPanel panelGasolina = new JPanel();
			panelGasolina.setLayout(new BoxLayout(panelGasolina, BoxLayout.Y_AXIS));
			panelGasolina.add(radioGasolina);
			panelGasolina.add(new PanelEtiqYCompon("Tipo de Flujo: ", comboTipoFlujo));
			panelGasolina.add(new PanelEtiqYCompon("Tipo de Bobina: ", campoBobina));
			
			JPanel panelDiesel = new JPanel();
			panelDiesel.setLayout(new BoxLayout(panelDiesel, BoxLayout.Y_AXIS));
			panelDiesel.add(radioDiesel);
			panelDiesel.add(new PanelEtiqYCompon("Tipo de Bomba: ", campoTipoBomba));
			panelDiesel.add(new PanelEtiqYCompon("Tipo de Filtro: ", campoTipoFiltro));
			
			add(panelGasolina); add(panelDiesel);
			
			setBorder(BorderFactory.createCompoundBorder(
					BorderFactory.createTitledBorder("Tipo de Vehículo"),
					BorderFactory.createEmptyBorder(5, 5, 5, 5)));
		}
	}
	
	class PanelEtiqYCompon extends JPanel {
		public PanelEtiqYCompon(String etiq, Component compo) {
			add(new JLabel(etiq));
			add(compo);
			setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		}
	}
	
	// +++++ Clase ActionListener +++++
	class AccionElegirGasolina implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			seleccionarGasolina();
		}
	}
	
	class AccionElegirDiesel implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			seleccionarDiesel();
		}
	}

}
