package paneles;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.time.LocalDate;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import tallerVehiculos.Vehiculo;

public class PanelEstadoVehiculo extends JPanel{

	private Vehiculo vehiculo;
	public JButton botonIniciar;
	public JButton botonFinalizar;
	public JButton botonSalir;
	public JCheckBox checkEntregado;
	private JTextField campoInicio;
	private JTextField campoTermino;

	public PanelEstadoVehiculo() {
		iniciarComponentes();
		setLayout(new BorderLayout());
		add(new PanelNorte(), BorderLayout.NORTH);
		add(new PanelCentro(), BorderLayout.CENTER);
		add(new PanelSur(), BorderLayout.SOUTH);
	}
	
	private void iniciarComponentes() {
		vehiculo = null;
		botonIniciar = new JButton("<html><p style=\"text-align:center\">Iniciar<br>Reparación</p></html>");
		botonIniciar.setBackground(new Color(72, 193, 51));
		botonIniciar.setForeground(Color.WHITE);
		botonFinalizar = new JButton("<html><p style=\"text-align:center\">Finalizar<br>Reparación</p></html>");
		botonFinalizar.setBackground(Color.RED);
		botonFinalizar.setForeground(Color.WHITE);
		botonSalir = new JButton("SALIR");
		botonSalir.setFont(new Font("Verdana", Font.BOLD, 22));
		checkEntregado = new JCheckBox("Entregado");
		checkEntregado.setFont(new Font("Verdana", Font.BOLD, 18));
		//Los campos de texto no son editables para el usuario
		campoInicio = new JTextField(5);
		campoInicio.setEditable(false);
		campoTermino = new JTextField(5);
		campoTermino.setEditable(false);
	}

	public void setVehiculo(Vehiculo v) {
		vehiculo = v;
		
		campoInicio.setText(vehiculo.getFechaInicio());
		campoTermino.setText(vehiculo.getFechaTermino());
		
		checkEntregado.setSelected(vehiculo.isEntregado());
		
		//Decidimos que botones estarán activos
		if (vehiculo.isEntregado()) {
			//Vehiculo entregado, no se puede alterar su estado de reparación
			botonIniciar.setEnabled(false);
			botonFinalizar.setEnabled(false);
			//Pero si se puede revertir la entrega al cliente
			checkEntregado.setEnabled(true);
		}
		else {
			if (vehiculo.getFechaInicio().equals("No Iniciado")) {
				//Aun no se ha iniciado la reparación de este Vehiculo
				botonIniciar.setEnabled(true);
				botonFinalizar.setEnabled(false);
				checkEntregado.setEnabled(true);
			}
			else if (vehiculo.getFechaTermino().equals("No Terminado")){
				//Reparacion iniciada, pero no terminada
				botonIniciar.setEnabled(false);
				botonFinalizar.setEnabled(true);
				//No se puede entregar hasta que se termine la reparacion
				checkEntregado.setEnabled(false);
			}
			else {
				//Reparacion iniciada y terminada.
				//Aunque cabe la posibilidad de iniciar una nueva reparación
				botonIniciar.setEnabled(true);
				botonFinalizar.setEnabled(false);
				checkEntregado.setEnabled(true);
			}
		}
	}
	
	public void iniciar() {
		vehiculo.setFechaInicio(LocalDate.now());
		vehiculo.setFechaTermino(null);
		campoInicio.setText(vehiculo.getFechaInicio());
		campoTermino.setText(vehiculo.getFechaTermino());
		botonIniciar.setEnabled(false);
		botonFinalizar.setEnabled(true);
		checkEntregado.setEnabled(false);
	}
	
	public void finalizar() {
		vehiculo.setFechaTermino(LocalDate.now());
		campoInicio.setText(vehiculo.getFechaInicio());
		campoTermino.setText(vehiculo.getFechaTermino());
		botonIniciar.setEnabled(true);
		botonFinalizar.setEnabled(false);
		checkEntregado.setEnabled(true);
	}
	
	public void entregar() {
		/*
		 * Hay dos posibilidades, que el Vehículo NO está
		 * entregado y por lo tanto, se va a entregar ahora.
		 * La otra es que SI esté entregado, pero que por algún 
		 * motivo el vehículo regresa de nuevo al taller y por 
		 * lo tanto, dejará de estar Entregado.
		 *
		 * Consultamos el atributo "entregado"
		 * del objeto Vehículo que estamos gestionando. 
		 */
		
		if (vehiculo.isEntregado()) { //YA entregado, pero regresa al taller
			botonIniciar.setEnabled(true); //Se puede iniciar una nueva reparación
			botonFinalizar.setEnabled(false);
			vehiculo.setEntregado(false); //Deja de estar entregado
		}
		else {//No entregado, se entrega ahora al cliente
			botonIniciar.setEnabled(false); //Ya no se pueden iniciar reparaciones
			botonFinalizar.setEnabled(false);
			vehiculo.setEntregado(true); //Ahora consta como entregado.
		}
	}
	
	class PanelNorte extends JPanel {
		public PanelNorte() {
			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			JPanel panelTitulo = new JPanel();
			JLabel titulo = new JLabel("<html><p><u>ESTADO del Vehiculo</u></p></html>");
			titulo.setFont(new Font("Verdana", Font.BOLD, 32));
			panelTitulo.add(titulo);
			
			JPanel panelLinea1 = new JPanel();
			JLabel linea1 = new JLabel("Utilice los botones para Iniciar y Finalizar el estado de Reparación");
			linea1.setFont(new Font("Verdana", Font.PLAIN, 22));
			panelLinea1.add(linea1);
			
			JPanel panelLinea2 = new JPanel();
			JLabel linea2 = new JLabel("Marque y desmarque la casilla para indicar si el Vehículo ha sido entregado.");
			linea2.setFont(new Font("Verdana", Font.PLAIN, 22));
			panelLinea2.add(linea2);
			
			add(panelTitulo);
			add(panelLinea1);
			add(panelLinea2);
			setBorder(BorderFactory.createCompoundBorder(
					BorderFactory.createEmptyBorder(20, 20, 10, 20),
					BorderFactory.createCompoundBorder(BorderFactory.createRaisedBevelBorder(),
							BorderFactory.createEmptyBorder(10, 10, 10, 10))));
		}
	}
	
	class PanelCentro extends JPanel {
		public PanelCentro() {
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			JPanel inicio = new JPanel();
			inicio.add(new PanelCampoYboton(campoInicio, botonIniciar));
			JPanel terminar = new JPanel();
			terminar.add(new PanelCampoYboton(campoTermino, botonFinalizar));
			JPanel entregado = new JPanel();
			entregado.add(checkEntregado);
			entregado.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
			
			add(inicio); add(terminar); add(entregado);
			setBorder(BorderFactory.createEmptyBorder(10, 20, 10, 20));
		}
	}
	
	class PanelCampoYboton extends JPanel {
		public PanelCampoYboton(JTextField campo, JButton boton) {
			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			campo.setHorizontalAlignment(SwingConstants.CENTER);
			campo.setFont(new Font("Verdana", Font.BOLD, 18));
			boton.setFont(new Font("Verdana", Font.BOLD, 22));
			add(campo);
			add(boton);
			setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		}
	}
	
	class PanelSur extends JPanel {
		public PanelSur() {
			add(botonSalir);
			setBorder(BorderFactory.createEmptyBorder(10, 20, 20, 20));
		}
	}

}