package tallerVehiculos;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;

import javax.swing.JOptionPane;

import paneles.*;

public class Controlador {

	//Datos Modelo
	private String nombreLogueado;
	private GestionUsuario gestorUsuarios;
	private GestionVehiculos gestorVehiculos;
	//Datos Vista
	private Taller taller; //JFrame principal
	//Paneles para la Vista
	private UsuarioLogin panelLogin;
	private PanelDatosVehiculo panelDatVehi;
	private PanelEstadoVehiculo panelEstadoVehi;

	public Controlador (Taller t) {
		taller = t;
		iniciarGestorUsuarios();
		iniciarGestorVehiculos();
		panelLogin = new UsuarioLogin();
		panelDatVehi = new PanelDatosVehiculo();
		panelEstadoVehi = new PanelEstadoVehiculo();
		asignarAcciones();
		mostrarPanelLogin();
	}

	private void asignarAcciones() {
		panelLogin.botonCrear.addActionListener(new AccionCrearUsuario());
		panelLogin.botonIniciar.addActionListener(new AccionIniciarSesion());
		
		panelDatVehi.botonCancelar.addActionListener(new AccionCancelarVehiculo());
		panelDatVehi.botonAceptar.addActionListener(new AccionAceptarVehiculo());
		
		panelEstadoVehi.botonIniciar.addActionListener(new AccionIniciarReparacion());
		panelEstadoVehi.botonFinalizar.addActionListener(new AccionFinalizarReparacion());
		panelEstadoVehi.checkEntregado.addActionListener(new AccionEntregarVehiculo());
		panelEstadoVehi.botonSalir.addActionListener(new AccionSalirGestion());
	}

	/**
	 * Establece en el marco principal que muestre
	 * el panel de inicio de sesion de usuarios.
	 */
	public void mostrarPanelLogin() {
		panelLogin.reset();
		taller.setContentPane(panelLogin);
		taller.pack();
		taller.setLocationRelativeTo(null);
	}

	/**
	 * Intentará recuperar los datos de gestion de
	 * usuarios guardados en disco.<br>Si no puede
	 * recuperarlos, ya sea porque no existe el archivo
	 * o porque no tiene acceso a él o está corrupto,
	 * iniciará un Gestor de Usuarios nuevo.
	 */
	private void iniciarGestorUsuarios() {
		File usuarios = new File("usuarios.bin");
		if (usuarios.exists()) {
			ObjectInputStream ois;
			try {
				ois = new ObjectInputStream(new FileInputStream(usuarios));
				Object datos = ois.readObject();
				if (datos instanceof GestionUsuario) {
					gestorUsuarios = (GestionUsuario)datos;
				}
				else {
					mensajeEmergente("Datos de usuarios guardados no son válidos", "Gestion Usuarios");
					gestorUsuarios = new GestionUsuario();
				}
				ois.close();
			} catch (IOException e) {
				mensajeEmergente("No se pudo acceder a:\n" + usuarios.getAbsolutePath(), "Gestion Usuarios");
				gestorUsuarios = new GestionUsuario();
			} catch (ClassNotFoundException e) {
				mensajeEmergente("Datos de usuarios guardados no son válidos", "Gestion Usuarios");
				gestorUsuarios = new GestionUsuario();
			}

		}
		else {
			mensajeEmergente("No hay datos de Usuarios guardados", "Gestion Usuarios");
			gestorUsuarios = new GestionUsuario();
		}
	}
	
	private void iniciarGestorVehiculos() {
		File vehiculos = new File("vehiculos.bin");
		if (vehiculos.exists()) {
			ObjectInputStream ois;
			try {
				ois = new ObjectInputStream(new FileInputStream(vehiculos));
				Object datos = ois.readObject();
				if (datos instanceof GestionVehiculos) {
					gestorVehiculos = (GestionVehiculos)datos;
				}
				else {
					mensajeEmergente("Datos de Vehículos guardados no son válidos", "Gestión Vehículos");
					gestorVehiculos = new GestionVehiculos();
				}
				ois.close();
			} catch (IOException e) {
				mensajeEmergente("No se pudo acceder a:\n" + vehiculos.getAbsolutePath(), "Gestión Vehículos");
				gestorVehiculos = new GestionVehiculos();
			} catch (ClassNotFoundException e) {
				mensajeEmergente("Datos de Vehículos guardados no son válidos", "Gestión Vehículos");
				gestorVehiculos = new GestionVehiculos();
			}
		}
		else {
			mensajeEmergente("No hay datos de Vehículos guardados", "Gestión Vehículos");
			gestorVehiculos = new GestionVehiculos();
		}
	}

	// ++++++++++++++++++++ Clases Action Listener  ++++++++++++++++++++
	class AccionCrearUsuario implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			if (panelLogin.campoNombre.getText().isEmpty() || panelLogin.campoPassword.getPassword().length == 0)
				mensajeEmergente("Ha de rellenar los dos campos", "Crear Usuario");
			else {
				String nombre = panelLogin.campoNombre.getText();
				String password = String.copyValueOf(panelLogin.campoPassword.getPassword());
				String confirm = JOptionPane.showInputDialog(null, "Por favor, confirme el Password:");
				if (password.equals(confirm)) {
					if (gestorUsuarios.nuevoUsuario(new Usuario(nombre, password))) {
						mensajeEmergente("Nuevo usuario creado.\nSesión iniciada",
								"Crear Usuario");
						gestorUsuarios.guardarUsuarios();
						taller.usuarioLogueado = true;
						nombreLogueado = nombre; //Guardamos nombre del usuario logueado
						taller.setContentPane(new PanelBienvenida(nombreLogueado));
						taller.pack();
						taller.setLocationRelativeTo(null);
					}
				}
				else
					mensajeEmergente("El password de registro no coincide", "Crear Usuario");
			}

		}
	}

	class AccionIniciarSesion implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			if (panelLogin.campoNombre.getText().isEmpty() || panelLogin.campoPassword.getPassword().length == 0)
				mensajeEmergente("Ha de rellenar los dos campos", "Crear Usuario");
			else {
				String nombre = panelLogin.campoNombre.getText();
				String password = String.copyValueOf(panelLogin.campoPassword.getPassword());
				if (gestorUsuarios.validarLogin(new Usuario(nombre, password))) {
					taller.usuarioLogueado = true;
					nombreLogueado = nombre;
					taller.setContentPane(new PanelBienvenida(nombreLogueado));
					taller.pack();
					taller.setLocationRelativeTo(null);
				}
			}
		}
	}

	class AccionCerrarSesion implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			if (taller.usuarioLogueado) {
				taller.usuarioLogueado = false;
				mostrarPanelLogin();
			}
			else
				mensajeEmergente("Ha de iniciar sesión para acceder a esta función.",
						"Opción no disponible");
		}
	}

	class AccionAgregarVehiculo implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			if (taller.usuarioLogueado) {
				panelDatVehi.resetCampos();
				taller.setContentPane(panelDatVehi);
				taller.pack();
				taller.setLocationRelativeTo(null);
			}
			else
				mensajeEmergente("Ha de iniciar sesión para acceder a esta función.",
						"Opción no disponible");
		}
	}
	
	class AccionCancelarVehiculo implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			taller.setContentPane(new PanelBienvenida(nombreLogueado));
			taller.pack();
			taller.setLocationRelativeTo(null);
		}
	}
	
	class AccionAceptarVehiculo implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			if (panelDatVehi.editando) {
				if (panelDatVehi.formularioEsValido()) {
					panelDatVehi.aceptarEdicion();
					mensajeEmergente("Los datos se han modificado", "Editar Vehículo");
					gestorVehiculos.guardarVehiculos();
				}
				else
					mensajeEmergente("Faltan campos por rellenar.", "Nuevo Vehículo");
			}
			else { //Estamos creando nuevo Vehículo
				if (panelDatVehi.formularioEsValido()) {
					if (gestorVehiculos.nuevoVehiculo(panelDatVehi.getNuevoVehiculo())) {
						mensajeEmergente("Nuevo Vehículo registrado", "Nuevo Vehículo");
						gestorVehiculos.guardarVehiculos();
					}
				}
				else
					mensajeEmergente("Faltan campos por rellenar.", "Nuevo Vehículo");
			}
		}
	}
	class AccionEditarVehiculo implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			String dato = JOptionPane.showInputDialog(null,
					"Introduzca placa o VIN del Vehículo que quiere EDITAR",
					"Editar Vehículo", JOptionPane.QUESTION_MESSAGE);
			if (dato != null) {
				Vehiculo v = gestorVehiculos.buscarVehiculo(dato);
				if (v == null)
					mensajeEmergente("No hay Vehículos con esa placa o VIN",
							"Editar Vehículo");
				else {
					panelDatVehi.editarVehiculo(v);
					taller.setContentPane(panelDatVehi);
					taller.pack();
					taller.setLocationRelativeTo(null);
				}
			}
		}
	}
	
	class AccionEliminarVehiculo implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			String dato = JOptionPane.showInputDialog(null,
					"Introduzca placa o VIN del Vehículo que quiere ELIMINAR",
					"Eliminar Vehículo", JOptionPane.QUESTION_MESSAGE);
			if (dato != null) {
				Vehiculo v = gestorVehiculos.buscarVehiculo(dato);
				if (v == null)
					mensajeEmergente("No hay Vehículos con esa placa o VIN",
							"Eliminar Vehículo");
				else {
					int confirm = JOptionPane.showConfirmDialog(null,
							"¿Seguro que quiere ELIMINAR vehículo con VIN: " + v.getVIN(),
							"Eliminar Vehículo", JOptionPane.YES_NO_OPTION,
							JOptionPane.QUESTION_MESSAGE);
					if (confirm == JOptionPane.YES_OPTION) {
						gestorVehiculos.eliminarVehiculo(v);
						gestorVehiculos.guardarVehiculos();
						mensajeEmergente("¡¡Vehículo Eliminado!!", "Eliminar Vehículo");
					}
				}
			}
		}
	}
	
	class AccionGestionarVehiculo implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			String dato = JOptionPane.showInputDialog(null,
					"Introduzca placa o VIN del Vehículo que quiere GESTIONAR",
					"Gestionar Vehículo", JOptionPane.QUESTION_MESSAGE);
			if (dato != null) {
				Vehiculo v = gestorVehiculos.buscarVehiculo(dato);
				if (v == null)
					mensajeEmergente("No hay Vehículos con esa placa o VIN",
							"Gestionar Vehículo");
				else {
					panelEstadoVehi.setVehiculo(v);
					taller.setContentPane(panelEstadoVehi);
					taller.pack();
					taller.setLocationRelativeTo(null);
				}
			}
		}
	}
	
	class AccionIniciarReparacion implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			panelEstadoVehi.iniciar();
			gestorVehiculos.guardarVehiculos();
		}
	}
	
	class AccionFinalizarReparacion implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			panelEstadoVehi.finalizar();
			gestorVehiculos.guardarVehiculos();
		}
	}
	
	class AccionEntregarVehiculo implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			panelEstadoVehi.entregar();
			gestorVehiculos.guardarVehiculos();
		}
	}
	
	class AccionSalirGestion implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			taller.setContentPane(new PanelBienvenida(nombreLogueado));
			taller.pack();
			taller.setLocationRelativeTo(null);
		}
	}

	/**
	 * Puesto que en este código se va a recurrir varias veces a mostrar
	 * JOptionPane informativos en pantalla, vale la pena simplificar el código
	 * creando este método para no repetirlo decenas de veces
	 * @param mensaje Texto informativo que mostrará el JOptionPane
	 * @param titulo Título descriptivo del JOptionPane
	 */
	private void mensajeEmergente(String mensaje, String titulo) {
		JOptionPane.showMessageDialog(null, mensaje, titulo, JOptionPane.WARNING_MESSAGE);
	}

}
